/*!
 * vue-form-container v1.0.2
 * (c) 2019-present Vitor Luiz Cavalcanti <vitorluizc@outlook.com> (https://vitorluizc.github.io)
 * Released under the MIT License.
 */
'use strict';

var valite = require('valite');

/**
 * Creates a clone of object only with keys, without values.
 * @param {object} object
 * @returns {object}
 */
var cloneOnlyKeys = function (object) {
  var clone = Object.create(null);
  Object.keys(object).forEach(function (key) { return clone[key] = undefined; });
  return clone;
};
/**
 * Defines acessors (get and set) to key on object.
 * @param {object} object
 * @param {string} key
 * @param {{ get: () => any, set: (value: any) => void }} acessors
 */

var definePropertyAcessors = function (object, key, ref) {
  var get = ref.get;
  var set = ref.set;

  return Object.defineProperty(object, key, {
  get: get,
  set: set,
  enumerable: true,
  configurable: false
});
};

/**
 * A function to check if value attends to predicate.
 * @callback Predicate
 * @param {*} value Any value could be checked.
 * @returns {boolean} `true` if value attends to predicate.
 */

/**
 * Value returned by `typeof` to determine what is value's type.
 * @typedef {('number' | 'object' | 'string' | 'symbol' | 'boolean' | 'function' | 'undefined')} Type
 */

/**
 * Check if value is of type.
 * @param {Type} type
 * @returns {Predicate}
 */
var isType = function (type) { return function (value) { return typeof value === type; }; };
/**
 * Check if value is an object.
 * @param {*} value
 * @returns {boolean}
 */

var isObject = function (value) { return !!value && isType('object')(value); };
/**
 * Check if value is an Array and every element attends the predicate.
 * @param {Predicate} predicate
 * @returns {Predicate}
 */

var isEvery = function (predicate) { return function (value) { return Array.isArray(value) && value.every(predicate); }; };
/**
 * Check if every object properties attends the predicate.
 * @param {Predicate} predicate
 * @returns {Predicate}
 */

var isEveryProperty = function (predicate) { return function (value) {
  if (!isObject(value)) { return false; }
  var properties = Object.keys(value).map(function (key) { return value[key]; });
  return properties.every(isEvery(predicate));
}; };

/**
 * Model of global form accessor object.
 * @typedef {Object} FormAcessor
 * @property {Object.<string, *>} fields
 * @property {Object.<string, string>} errors
 * @property {function(string,*):void} update
 * @property {boolean} isValid
 * @property {boolean} isLoading
 * @property {function():Promise.<void>} validateForm
 * @property {function(string):Promise.<void>} validateField
 */

var script = {
  props: {
    schema: {
      type: Object,
      required: true,
      validator: isEveryProperty(isType('function'))
    },
    initial: {
      type: Object,
      default: function () { return Object.create(null); },
      validator: isObject
    }
  },

  data: function data() {
    return {
      ticks: 0,
      isInitialized: false,
      errors: Object.create(null),
      values: Object.create(null)
    };
  },

  watch: {
    schema: {
      /**
       * On schema changes it update `values` state. It merges only keys in
       * schema and outdated `values`, or `initial` if FormContainer is
       * not initialized.
       * @param {object} schema
       */
      handler: function handler(schema) {
        var values = this.isInitialized ? this.values : this.initial;
        if (this.isInitialized) { this.isInitialized = true; }
        this.values = Object.assign(cloneOnlyKeys(schema), values);
      },

      immediate: true
    }
  },
  computed: {
    fields: function fields() {
      var this$1 = this;

      var fields = Object.create(null);
      Object.keys(this.values).forEach(function (field) { return definePropertyAcessors(fields, field, {
        get: function () { return this$1.values[field]; },
        set: function (value) { return this$1.update(field, value); }
      }); });
      return fields;
    },

    isLoading: function isLoading() {
      return this.ticks > 0;
    },

    isValid: function isValid$1() {
      return !this.isLoading && valite.isValid(this.errors);
    }

  },
  methods: {
    /**
     * Updates a field and trigger its validation.
     * @param {string} field Field's name.
     * @param {*} value Field's value.
     */
    update: function update(field, value) {
      this.$set(this.values, field, value);
      this.validateField(field);
    },

    /**
     * Validates a field and updates Form properties.
     * @param {string} field Field's name.
     * @returns {Promise.<void>}
     */
    validateField: function validateField(field) {
      var this$1 = this;

      this.ticks += 1;
      return valite.validate(this.values[field], this.schema[field]).then(function (error) {
        this$1.$set(this$1.errors, field, error);
        this$1.ticks -= 1;
      }).catch(function (_) {
        console.error(("Can't validate field \"" + field + "\"."));
        this$1.ticks -= 1;
      });
    },

    /**
     * Validates all fields and updates Form properties.
     * @returns {Promise.<void>}
     */
    validateForm: function validateForm() {
      var this$1 = this;

      this.ticks += 1;
      return valite.validateObject(this.values, this.schema).then(function (errors) {
        this$1.errors = errors;
        this$1.ticks -= 1;
      }).catch(function (_) {
        console.error('Can\'t validate form.');
        this$1.ticks -= 1;
      });
    }

  },

  render: function render() {
    return this.$scopedSlots.default(this);
  }

};

/* script */
            var __vue_script__ = script;
            
/* template */

  /* style */
  var __vue_inject_styles__ = undefined;
  /* scoped */
  var __vue_scope_id__ = undefined;
  /* module identifier */
  var __vue_module_identifier__ = undefined;
  /* functional template */
  var __vue_is_functional_template__ = undefined;
  /* component normalizer */
  function __vue_normalize__(
    template, style, script$$1,
    scope, functional, moduleIdentifier,
    createInjector, createInjectorSSR
  ) {
    var component = (typeof script$$1 === 'function' ? script$$1.options : script$$1) || {};

    // For security concerns, we use only base name in production mode.
    component.__file = "FormContainer.vue";

    if (!component.render) {
      component.render = template.render;
      component.staticRenderFns = template.staticRenderFns;
      component._compiled = true;

      if (functional) { component.functional = true; }
    }

    component._scopeId = scope;

    return component
  }
  /* style inject */
  
  /* style inject SSR */
  

  
  var FormContainer = __vue_normalize__(
    {},
    __vue_inject_styles__,
    __vue_script__,
    __vue_scope_id__,
    __vue_is_functional_template__,
    __vue_module_identifier__,
    undefined,
    undefined
  );

module.exports = FormContainer;
